'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; /* eslint no-underscore-dangle: 0 */
/* eslint no-param-reassign: 0 */
/* eslint no-use-before-define: 0 */

var _denormalizr = require('denormalizr');

function mergeMappings(entity, mapObj) {
  if (!mapObj) return entity;

  if (typeof mapObj === 'function') {
    return mapObj(entity);
  }

  if (Object.keys(mapObj).includes('id')) {
    return Object.assign({}, entity, mapObj);
  }

  return Object.assign({}, mapObj[entity.id], entity);
}

function iterateOverObject(object, mappings) {
  if (mappings === {}) return object;
  var mappingsKeys = Object.keys(mappings || {});

  Object.keys(object).forEach(function (key) {
    var isArray = Array.isArray(object[key]);

    if (mappingsKeys.includes(key)) {
      if (isArray) {
        object[key] = object[key].map(function (item) {
          return mergeMappings(item, mappings[key]);
        });
      } else {
        object[key] = mergeMappings(object[key], mappings[key]);
      }
    }

    if (isArray) {
      object[key] = iterateOverObject(object[key], mappings);
    } else if (_typeof(object[key]) === 'object') {
      object[key] = iterateOverObject(object[key], mappings);
    }
  });

  return object;
}

exports.default = function (entity, entities, schema, mappings) {
  var entityKeys = Object.keys(entity);

  // entity ~ { id: 1, isLoading: false }
  if ((typeof entity === 'undefined' ? 'undefined' : _typeof(entity)) === 'object' && entityKeys.includes('id')) {
    var state = (0, _denormalizr.denormalize)(entity.id, entities, schema);
    state = iterateOverObject(state, mappings);
    state = mergeMappings(state, entity);
    return mappings ? mergeMappings(state, mappings[schema._key]) : state;

    // entity ~ [1, 2]
  } else if (Array.isArray(entity) && (typeof entity[0] === 'number' || typeof entity[0] === 'string')) {
    var _state = (0, _denormalizr.denormalize)(entity, entities, schema);
    _state = iterateOverObject(_state, mappings);

    if (mappings && schema._itemSchema._key in mappings) {
      return _state.map(function (item) {
        return mergeMappings(item, mappings[schema._itemSchema._key]);
      });
    }

    return _state.map(function (item) {
      item = mergeMappings(item, entity.find(function (i) {
        return i === item.id;
      }));
      return mappings ? mergeMappings(item, mappings[schema._itemSchema._key]) : item;
    });

    // entity ~ { 1: { isLoading: false } }
  } else if ((typeof entity === 'undefined' ? 'undefined' : _typeof(entity)) === 'object') {
    var _state2 = (0, _denormalizr.denormalize)(entityKeys.map(function (id) {
      return id;
    }), entities, schema);
    _state2 = iterateOverObject(_state2, mappings);
    return _state2.map(function (item) {
      return mergeMappings(entity[item.id], item);
    });

    // entity ~ 1
  } else if (typeof entity === 'number' || typeof entity === 'string') {
    var _state3 = (0, _denormalizr.denormalize)(entity, entities, schema);
    _state3 = iterateOverObject(_state3, mappings);
    return mappings ? mergeMappings(_state3, mappings[schema._key]) : _state3;
  }

  // fall back to plain old denormalize
  return (0, _denormalizr.denormalize)(entity, entities, schema);
};